@@ -40,11 +40,11 @@ class API(api.API, metaclass=abc.ABCMeta):
     def ls_del(self, switch, if_exists=False):
         """Delete logical switch 'switch' and all its ports
 
-        :param switch:   The name or uuid of the switch
-        :type switch:    string or uuid.UUID
-        :type if_exists: If True, don't fail if the switch doesn't exist
-        :type if_exists: boolean
-        :returns:        :class:`Command` with no result
+        :param switch:    The name or uuid of the switch
+        :type switch:     string or uuid.UUID
+        :param if_exists: If True, don't fail if the switch doesn't exist
+        :type if_exists:  boolean
+        :returns:         :class:`Command` with no result
         """
 
     @abc.abstractmethod
@@ -96,6 +96,17 @@ class API(api.API, metaclass=abc.ABCMeta):
         :returns: :class:`Command` with RowView result
         """
 
+    @abc.abstractmethod
+    def ls_get_localnet_ports(self, switch, if_exists=False):
+        """Get localnet ports on logical switch 'switch'
+
+        :param switch:    The name or uuid of the switch
+        :type switch:     string or uuid.UUID
+        :param if_exists: If True, don't fail if the switch doesn't exist
+        :type if_exists:  boolean
+        :returns: :class:`Command` with the RowView list result
+        """
+
     @abc.abstractmethod
     def acl_add(self, switch, direction, priority, match, action, log=False):
         """Add an ACL to 'switch'
@@ -117,7 +128,8 @@ class API(api.API, metaclass=abc.ABCMeta):
         """
 
     @abc.abstractmethod
-    def acl_del(self, switch, direction=None, priority=None, match=None):
+    def acl_del(self, switch, direction=None, priority=None, match=None,
+                if_exists=False):
         """Remove ACLs from 'switch'
 
         If only switch is supplied, all the ACLs from the logical switch are
@@ -133,6 +145,9 @@ class API(api.API, metaclass=abc.ABCMeta):
         :type priority:   int
         :param match:     The match rule
         :type match:      string
+        :param if_exists: If True, don't fail if the parent logical switch
+                          containing the ACL doesn't exist
+        :type if_exists:  boolean
         :returns:         :class:`Command` with no result
         """
 
@@ -178,7 +193,7 @@ class API(api.API, metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
     def pg_acl_del(self, port_group, direction=None, priority=None,
-                   match=None):
+                   match=None, if_exists=False):
         """Remove ACLs from 'port_group'
 
         If only port_group is supplied, all the ACLs from the logical switch
@@ -194,6 +209,9 @@ class API(api.API, metaclass=abc.ABCMeta):
         :type priority:    int
         :param match:      The match rule
         :type match:       string
+        :param if_exists:  If True, don't fail if the parent port group
+                           containing the ACL doesn't exist
+        :type if_exists:   boolean
         :returns:          :class:`Command` with no result
         """
 
@@ -639,14 +657,17 @@ class API(api.API, metaclass=abc.ABCMeta):
         """
 
     @abc.abstractmethod
-    def lrp_set_options(self, port, **options):
+    def lrp_set_options(self, port, if_exists=False, **options):
         """Set options related to the type of 'port'
 
-        :param port:    The name or uuid of the port
-        :type port:     string or uuid.UUID
-        :param options: keys and values for the port 'options' dict
-        :type options:  key: string, value: string
-        :returns:       :class:`Command` with no result
+        :param port:      The name or uuid of the port
+        :type port:       string or uuid.UUID
+        :param options:   keys and values for the port 'options' dict
+        :param if_exists: If True, don't fail if the logical router port
+                          doesn't exist
+        :type if_exists:  boolean
+        :type options:    key: string, value: string
+        :returns:         :class:`Command` with no result
         """
 
     @abc.abstractmethod
@@ -695,6 +716,38 @@ class API(api.API, metaclass=abc.ABCMeta):
         :returns:               :class:`Command` with no result
         """
 
+    @abc.abstractmethod
+    def lrp_set_ha_chassis_group(self, port, ha_chassis_group):
+        """Set HA chassis group for 'port'
+
+        :param port:             The name or uuid of the port
+        :type port:              string or uuid.UUID
+        :param ha_chassis_group: The name or uuid of the HA chassis group
+        :type ha_chassis_group:  string or uuid.UUID
+        :returns:                :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def lrp_get_ha_chassis_group(self, port):
+        """Get the HA chassis group on 'port'
+
+        :param port: The name or uuid of the port
+        :type port:  string or uuid.UUID
+        :returns:    :class:`Command` with RowView result or None
+        """
+
+    @abc.abstractmethod
+    def lrp_del_ha_chassis_group(self, port, if_exists=False):
+        """Delete HA chassis group from 'port'
+
+        :param port:      The name or uuid of the port
+        :type port:       string or uuid.UUID
+        :param if_exists: If True, don't fail if the HA chassis group
+                          doesn't exist
+        :type if_exists:  boolean
+        :returns:         :class:`Command` with no result
+        """
+
     @abc.abstractmethod
     def lrp_add_networks(self, port, networks, may_exist=False):
         """Add a network to 'port'
@@ -724,7 +777,8 @@ class API(api.API, metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
     def lr_route_add(self, router, prefix, nexthop, port=None,
-                     policy='dst-ip', may_exist=False):
+                     policy='dst-ip', may_exist=False, ecmp=False,
+                     route_table=const.MAIN_ROUTE_TABLE, bfd=None):
         """Add a route to 'router'
 
         :param router:    The name or uuid of the router
@@ -743,11 +797,20 @@ class API(api.API, metaclass=abc.ABCMeta):
         :type policy:     string, 'dst-ip' or 'src-ip'
         :param may_exist: If True, don't fail if the route already exists
         :type may_exist:  boolean
+        :param ecmp:      Enable ECMP support. If True adding routes with
+                          same IP prefix is allowed as long as the nexthop is
+                          different
+        :type ecmp:       boolean
+        :param route_table: The name of route table
+        :type route_table:  str
+        :param bfd:       A reference to the BFD table row.
+        :type bfd:        uuid.UUID
         returns:          :class:`Command` with RowView result
         """
 
     @abc.abstractmethod
-    def lr_route_del(self, router, prefix=None, if_exists=False):
+    def lr_route_del(self, router, prefix=None, if_exists=False, nexthop=None,
+                     route_table=const.MAIN_ROUTE_TABLE):
         """Remove routes from 'router'
 
         :param router:    The name or uuid of the router
@@ -756,16 +819,25 @@ class API(api.API, metaclass=abc.ABCMeta):
         :type prefix:     type string
         :param if_exists: If True, don't fail if the port doesn't exist
         :type if_exists:  boolean
+        :parm nexthop:    The gateway to use for this route, which should be
+                          the IP address of one of `router`'s logical router
+                          ports or the IP address of a logical port
+        :type nexthop:    string
+        :param route_table: The name of route table
+        :type route_table:  str
         :returns:        :class:`Command` with no result
         """
 
     @abc.abstractmethod
-    def lr_route_list(self, router):
+    def lr_route_list(self, router, route_table=None):
         """Get the UUIDs of static logical routes from 'router'
 
-        :param router: The name or uuid of the router
-        :type router:  string or uuid.UUID
-        :returns:      :class:`Command` with RowView list result
+        :param router:      The name or uuid of the router
+        :type router:       string or uuid.UUID
+        :param route_table: The name of route table. Pass "" to get routes of
+                            global route table only
+        :type route_table:  str
+        :returns:           :class:`Command` with RowView list result
         """
 
     @abc.abstractmethod
@@ -1198,6 +1270,19 @@ class API(api.API, metaclass=abc.ABCMeta):
         :returns:    :class:`Command` with no result
         """
 
+    @abc.abstractmethod
+    def dns_set_options(self, uuid, if_exists=True, **options):
+        """Sets the 'options' field of the DNS row
+
+        :param uuid: The uuid of the DNS row to set the options with
+        :type uuid:  string or uuid.UUID
+        :param if_exists: Do not fail if the DNS row does not exist
+        :type if_exists:  boolean
+        :param options: keys and values for the DNS 'options' dict
+        :type options:  key: string, value: string
+        :returns:    :class:`Command` with no result
+        """
+
     @abc.abstractmethod
     def pg_add(self, name=None, may_exist=False, **columns):
         """Create a port group
@@ -1386,3 +1471,140 @@ class API(api.API, metaclass=abc.ABCMeta):
         :type meter:   string or uuid.UUID
         :returns:      :class:`Command` with RowView result
         """
+
+    @abc.abstractmethod
+    def bfd_add(self, logical_port, dst_ip, min_tx=None, min_rx=None,
+                detect_mult=None, external_ids=None, options=None,
+                may_exist=False):
+        """Create a BFD entry
+
+        :param logical_port: Name of logical port where BFD engine should run.
+        :type logical_port:  str
+        :param dst_ip:       BFD peer IP address.
+        :type dst_ip:        str
+        :param min_tx:       This is the minimum interval, in milliseconds,
+                             that the local system would like to use when
+                             transmitting BFD Control packets, less any jitter
+                             applied. The value zero is reserved. Default value
+                             is 1000 ms.
+        :type min_tx:        Optional[int]
+        :param min_rx:       This is the minimum interval, in milliseconds,
+                             between received BFD Control packets that this
+                             system is capable of supporting, less any jitter
+                             applied by the sender. If this value is zero, the
+                             transmitting system does not want the remote
+                             system to send any periodic BFD Control packets.
+        :type min_rx:        Optional[int]
+        :param detect_mult:  Detection time multiplier.  The negotiated
+                             transmit interval, multiplied by this value,
+                             provides the Detection Time for the receiving
+                             system in Asynchronous mode. Default value is 5.
+        :type detect_mult:   Optional[int]
+        :param external_ids: Values to be added as external_id pairs.
+        :type external_ids:  Optional[Dict[str,str]]
+        :param options:      Reserved for future use.
+        :type options:       Optional[Dict[str,str]]
+        :param may_exist:    If True, update any existing BFD entry if it
+                             already exists.  Default is False which will raise
+                             an error if a BFD entry with same logical_port,
+                             dst_ip pair already exists.
+        :type may_exist:     Optional[bool]
+        :returns:            :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def bfd_del(self, uuid):
+        """Delete a BFD entry
+
+        :param uuid:   The uuid of the BFD entry
+        :type uuid:    uuid.UUID
+        :returns:      :class:`Command` with no result
+        """
+
+    @abc.abstractmethod
+    def bfd_find(self, logical_port, dst_ip):
+        """Find BFD entry.
+
+        :param logical_port: Name of logical port where BFD engine runs.
+        :type logical_port:  str
+        :param dst_ip:       BFD peer IP address.
+        :type dst_ip:        str
+        :returns:            :class:`Command` with List[Dict[str,any]] result
+        """
+
+    @abc.abstractmethod
+    def bfd_get(self, uuid):
+        """Get the BFD entry
+
+        :param uuid:   The uuid of the BFD entry
+        :type uuid:    uuid.UUID
+        :returns:      :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def mirror_get(self, uuid):
+        """Get the Mirror entry"""
+
+    @abc.abstractmethod
+    def mirror_del(self, mirror):
+        """Delete a Mirror"""
+
+    @abc.abstractmethod
+    def mirror_add(self, name, mirror_type, index, direction_filter, dest,
+                   external_ids=None,
+                   may_exist=False):
+        """Create a Mirror entry
+
+        :param name:    Name of the Mirror to create.
+        :type name:     str
+        :param mirror_type:    The type of the mirroring can be gre or erspan.
+        :type mirror_type:     str
+        :param index:   The index filed will be used for the Index field in
+                        ERSPAN header as decimal, and as hexadecimal value in
+                        the SpanID field, for GRE mirrors it will be the Key
+                        field.
+        :type index:    int
+        :param direction_filter:  The direction of the traffic to be mirrored,
+                                  can be from-lport and to-lprt.
+        :type direction_filter:   str
+        :param dest:    The destination IP address of the mirroring.
+        :type dest:     str
+
+
+        :param external_ids: Values to be added as external_id pairs.
+        :type external_ids:  Optional[Dict[str,str]]
+        :param may_exist:    If True, update any existing Mirror entry if it
+                             already exists.  Default is False which will raise
+                             an error if a Mirror entry with same logical_port,
+                             sink pair already exists.
+        :type may_exist:     Optional[bool]
+        :returns:            :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def lsp_attach_mirror(self, port, mirror, may_exist=False):
+        """Attaches an lsp to the given mirror
+
+        :param port: the id of the lsp
+        :type port: str
+        :param mirror: the name or ID of the mirror.
+        :type mirror: str
+        :param may_exist:    If True, don't fail if the mirror_rule already
+                             exists.
+        :type may_exist:     Optional[bool]
+        :returns: :class:`Command` with RowView result
+        """
+
+    @abc.abstractmethod
+    def lsp_detach_mirror(self, port, mirror, if_exist=False):
+        """Detaches an lsp from the given mirror
+
+        :param port: the id of the lsp
+        :type port: str
+        :param mirror: the name or ID of the mirror
+        :type mirror: str
+        :param if_exist:    If True, don't fail if the mirror_rules entry
+                            doesn't exist.
+        :type if_exist:     Optional[bool]
+        :returns: :class:`Command` with RowView result
+        """
