@@ -190,13 +190,14 @@ class PgAclAddCommand(_AclAddHelper):
 
 class _AclDelHelper(cmd.BaseCommand):
     def __init__(self, api, entity, direction=None,
-                 priority=None, match=None):
+                 priority=None, match=None, if_exists=False):
         if (priority is None) != (match is None):
             raise TypeError("Must specify priority and match together")
         if priority is not None and not direction:
             raise TypeError("Cannot specify priority/match without direction")
         super().__init__(api)
         self.entity = entity
+        self.if_exists = if_exists
         self.conditions = []
         if direction:
             self.conditions.append(('direction', '=', direction))
@@ -206,7 +207,14 @@ class _AclDelHelper(cmd.BaseCommand):
                                     ('match', '=', match)]
 
     def run_idl(self, txn):
-        entity = self.api.lookup(self.lookup_table, self.entity)
+        try:
+            entity = self.api.lookup(self.lookup_table, self.entity)
+        except idlutils.RowNotFound as e:
+            if self.if_exists:
+                return
+            msg = "%s %s does not exist" % (self.lookup_table, self.entity)
+            raise RuntimeError(msg) from e
+
         for acl in [a for a in entity.acls
                     if idlutils.row_match(a, self.conditions)]:
             entity.delvalue('acls', acl)
@@ -216,12 +224,6 @@ class _AclDelHelper(cmd.BaseCommand):
 class AclDelCommand(_AclDelHelper):
     lookup_table = 'Logical_Switch'
 
-    def __init__(self, api, switch, direction=None,
-                 priority=None, match=None):
-        # NOTE: we're overriding the constructor here to not break any
-        # existing callers before we introduced Port Groups.
-        super().__init__(api, switch, direction, priority, match)
-
 
 class PgAclDelCommand(_AclDelHelper):
     lookup_table = 'Port_Group'
@@ -903,6 +905,15 @@ class LrpAddCommand(cmd.BaseCommand):
                                            len(gwcs) - n, may_exist=True)
             cmd.run_idl(txn)
             lrp.addvalue('gateway_chassis', cmd.result)
+        ha_chassis_group = self.columns.pop('ha_chassis_group', None)
+        if ha_chassis_group:
+            try:
+                ha_chassis_group = self.api.lookup(
+                    'HA_Chassis_Group', ha_chassis_group)
+            except idlutils.RowNotFound:
+                raise RuntimeError(
+                    f"HA Chassis Group {ha_chassis_group} does not exist")
+            lrp.ha_chassis_group = ha_chassis_group.uuid
         self.set_columns(lrp, **self.columns)
         self.result = lrp.uuid
 
@@ -1017,6 +1028,56 @@ class LrpDelGatewayChassisCommand(cmd.BaseCommand):
             raise RuntimeError(msg)
 
 
+class LrpSetHaChassisGroupCommand(cmd.BaseCommand):
+    table = 'Logical_Router_Port'
+
+    def __init__(self, api, port, ha_chassis_group):
+        super().__init__(api)
+        self.port = port
+        self.ha_chassis_group = ha_chassis_group
+
+    def run_idl(self, txn):
+        lrp = self.api.lookup(self.table, self.port)
+        try:
+            hcg = self.api.lookup('HA_Chassis_Group', self.ha_chassis_group)
+        except idlutils.RowNotFound:
+            raise RuntimeError(
+                f"HA Chassis Group {self.ha_chassis_group} does not exist")
+        lrp.ha_chassis_group = hcg.uuid
+
+
+class LrpGetHaChassisGroupCommand(cmd.ReadOnlyCommand):
+    table = 'Logical_Router_Port'
+
+    def __init__(self, api, port):
+        super().__init__(api)
+        self.port = port
+
+    def run_idl(self, txn):
+        lrp = self.api.lookup(self.table, self.port)
+        if lrp.ha_chassis_group:
+            self.result = rowview.RowView(lrp.ha_chassis_group[0])
+        else:
+            self.result = None
+
+
+class LrpDelHaChassisGroupCommand(cmd.BaseCommand):
+    table = 'Logical_Router_Port'
+
+    def __init__(self, api, port, if_exists=False):
+        super().__init__(api)
+        self.port = port
+        self.if_exists = if_exists
+
+    def run_idl(self, txn):
+        lrp = self.api.lookup(self.table, self.port)
+        if lrp.ha_chassis_group:
+            lrp.ha_chassis_group = []
+        elif not self.if_exists:
+            msg = "HA chassis group on port %s does not exist" % self.port
+            raise RuntimeError(msg)
+
+
 class _LrpNetworksCommand(cmd.BaseCommand):
     table = 'Logical_Router_Port'
 
@@ -1254,7 +1315,7 @@ class LspDetachMirror(cmd.BaseCommand):
 class LrRouteAddCommand(cmd.BaseCommand):
     def __init__(self, api, router, prefix, nexthop, port=None,
                  policy='dst-ip', may_exist=False, ecmp=False,
-                 route_table=const.MAIN_ROUTE_TABLE):
+                 route_table=const.MAIN_ROUTE_TABLE, bfd=None):
         prefix = str(netaddr.IPNetwork(prefix))
         if nexthop != const.ROUTE_DISCARD:
             nexthop = str(netaddr.IPAddress(nexthop))
@@ -1266,6 +1327,7 @@ class LrRouteAddCommand(cmd.BaseCommand):
         self.policy = policy
         self.ecmp = ecmp
         self.route_table = route_table
+        self.bfd = bfd
         self.may_exist = may_exist
 
     def run_idl(self, txn):
@@ -1295,6 +1357,8 @@ class LrRouteAddCommand(cmd.BaseCommand):
         route.route_table = self.route_table
         if self.port:
             route.output_port = self.port
+        if self.bfd:
+            route.bfd = self.bfd
         lr.addvalue('static_routes', route)
         self.result = route.uuid
 
@@ -1476,7 +1540,7 @@ class LrPolicyAddCommand(cmd.AddCommand):
     table_name = 'Logical_Router_Policy'
 
     def __init__(self, api, router, priority, match, action, may_exist=False,
-                 **columns):
+                 chain=const.DEFAULT_CHAIN, **columns):
         if not 0 <= priority <= const.LR_POLICY_PRIORITY_MAX:
             raise ValueError("priority must be between 0 and %s, inclusive"
                              % (const.LR_POLICY_PRIORITY_MAX,))
@@ -1490,18 +1554,28 @@ class LrPolicyAddCommand(cmd.AddCommand):
                 action != const.POLICY_ACTION_REROUTE):
             raise ValueError("nexthop only valid for action %s"
                              % (const.POLICY_ACTION_REROUTE,))
+        if (action == const.POLICY_ACTION_JUMP and not
+                columns.get("jump_chain")):
+            raise ValueError("must specify jump_chain for action %s"
+                             % (const.POLICY_ACTION_JUMP,))
         super().__init__(api)
         self.router = router
         self.priority = priority
         self.match = match
         self.action = action
         self.may_exist = may_exist
+        self.chain = chain
         self.columns = columns
 
     def run_idl(self, txn):
         lr = self.api.lookup('Logical_Router', self.router)
         for policy in lr.policies:
-            if (self.priority, self.match) == (policy.priority, policy.match):
+            try:
+                policy_chain = policy.chain[0]
+            except (AttributeError, IndexError):
+                policy_chain = const.DEFAULT_CHAIN
+            if ((self.priority, self.match, self.chain) ==
+                    (policy.priority, policy.match, policy_chain)):
                 if self.may_exist:
                     policy.action = self.action
                     self.set_columns(policy, **self.columns)
@@ -1512,6 +1586,8 @@ class LrPolicyAddCommand(cmd.AddCommand):
         policy.priority = self.priority
         policy.match = self.match
         policy.action = self.action
+        if idlutils.table_has_column(self.api.idl, self.table_name, 'chain'):
+            policy.chain = [self.chain] if self.chain else []
         self.set_columns(policy, **self.columns)
         lr.addvalue('policies', policy)
         self.result = policy.uuid
@@ -1519,7 +1595,7 @@ class LrPolicyAddCommand(cmd.AddCommand):
 
 class LrPolicyDelCommand(cmd.BaseCommand):
     def __init__(self, api, router, priority=None, match=None,
-                 if_exists=False):
+                 if_exists=False, chain=const.DEFAULT_CHAIN):
         self.conditions = []
         if priority is not None:
             if not 0 <= priority <= const.LR_POLICY_PRIORITY_MAX:
@@ -1535,12 +1611,18 @@ class LrPolicyDelCommand(cmd.BaseCommand):
         self.priority = priority
         self.match = match
         self.if_exists = if_exists
+        self.chain = chain
 
     def run_idl(self, txn):
         lr = self.api.lookup('Logical_Router', self.router)
         found = False
         for policy in lr.policies:
-            if idlutils.row_match(policy, self.conditions):
+            try:
+                policy_chain = policy.chain[0]
+            except (AttributeError, IndexError):
+                policy_chain = const.DEFAULT_CHAIN
+            if (idlutils.row_match(policy, self.conditions) and
+                    policy_chain == self.chain):
                 found = True
                 lr.delvalue('policies', policy)
                 policy.delete()
@@ -1549,17 +1631,27 @@ class LrPolicyDelCommand(cmd.BaseCommand):
         if self.match and not (found or self.if_exists):
             raise RuntimeError(
                 "Policy with match %s and priority %s does not exist in "
-                "router %s" % (self.match, self.priority, self.router))
+                "router %s chain %s" % (self.match, self.priority,
+                                        self.router, self.chain))
 
 
 class LrPolicyListCommand(cmd.ReadOnlyCommand):
-    def __init__(self, api, router):
+    def __init__(self, api, router, chain=None):
         super().__init__(api)
         self.router = router
+        self.chain = chain
 
     def run_idl(self, txn):
         lr = self.api.lookup('Logical_Router', self.router)
-        self.result = [rowview.RowView(r) for r in lr.policies]
+        if self.chain is not None:
+            self.result = [rowview.RowView(r)
+                           for r in lr.policies
+                           if hasattr(r, 'chain') and (
+                           (r.chain and r.chain[0] == self.chain) or
+                           (not r.chain and
+                            self.chain == const.DEFAULT_CHAIN))]
+        else:
+            self.result = [rowview.RowView(r) for r in lr.policies]
 
 
 class LbAddCommand(cmd.BaseCommand):
@@ -1888,6 +1980,13 @@ class DnsSetExternalIdsCommand(cmd.BaseCommand):
             raise RuntimeError(msg) from e
 
 
+class DnsSetOptionsCommand(cmd.BaseSetOptionsCommand):
+    table = 'DNS'
+
+    def __init__(self, api, row_uuid, if_exists=True, **options):
+        super().__init__(api, row_uuid, if_exists=if_exists, **options)
+
+
 class PgAddCommand(cmd.AddCommand):
     table_name = 'Port_Group'
 
