@@ -10,9 +10,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from ovsdbapp.schema.ovn_northbound import impl_idl
 from ovsdbapp.tests.functional.schema import fixtures
 
 
+class NbApiFixture(fixtures.ApiImplFixture):
+    api_cls = impl_idl.OvnNbApiIdlImpl
+
+
 class LogicalSwitchFixture(fixtures.ImplIdlFixture):
     create = 'ls_add'
     delete = 'ls_del'
@@ -43,3 +48,13 @@ class DnsFixture(fixtures.ImplIdlFixture):
 class PortGroupFixture(fixtures.ImplIdlFixture):
     create = 'pg_add'
     delete = 'pg_del'
+
+
+class AddressSetFixture(fixtures.ImplIdlFixture):
+    create = 'address_set_add'
+    delete = 'address_set_del'
+
+
+class MeterFixture(fixtures.ImplIdlFixture):
+    create = 'meter_add'
+    delete = 'meter_del'
